"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addServerDependency = void 0;
const fs_1 = require("fs");
var NodeType;
(function (NodeType) {
    NodeType["OTHER"] = "OTHER";
    NodeType["GENESIS_OPENING_BLOCK"] = "GENESIS_OPENING_BLOCK";
    NodeType["DEPENDENCIES_OPENING_BLOCK"] = "DEPENDENCIES_OPENING_BLOCK";
    NodeType["DEPENDENCY"] = "DEPENDENCY";
})(NodeType || (NodeType = {}));
const parseSettings = (contents) => {
    const REGEX = {
        GENESIS_OPENING_BLOCK: /\s*genesis\s*{\s*/,
        DEPENDENCIES_OPENING_BLOCK: /\s*dependencies\s*{\s*/,
        DEPENDENCY: /\s*dependency\(.*\)\s*/,
    };
    let insideGenesisBlock = false;
    let insertionIndex;
    const lines = contents.split('\n').map((raw, index) => {
        let type = NodeType.OTHER;
        const meta = {};
        if (raw.match(REGEX.GENESIS_OPENING_BLOCK)) {
            type = NodeType.DEPENDENCIES_OPENING_BLOCK;
            insideGenesisBlock = true;
        }
        else if (raw.match(REGEX.DEPENDENCIES_OPENING_BLOCK) && insideGenesisBlock) {
            type = NodeType.DEPENDENCIES_OPENING_BLOCK;
            if (!insertionIndex) {
                insertionIndex = index + 1;
            }
        }
        else if (raw.match(REGEX.DEPENDENCY)) {
            type = NodeType.DEPENDENCY;
            const dep = raw.split(':');
            if (dep.length > 1) {
                meta.dependency = dep[1];
                meta.dependencyVersionProperty = dep[2].split('"')[1];
            }
        }
        return {
            raw,
            type,
            meta,
        };
    });
    return {
        lines,
        insertionIndex,
    };
};
const parseProperties = (contents, keyToUpsert) => {
    let insertionIndex;
    let updateIndex;
    const lines = contents.split('\n').map((raw, index) => {
        if (raw.includes('authVersion')) {
            insertionIndex = index + 1;
        }
        if (raw.includes(keyToUpsert)) {
            updateIndex = index;
        }
        return {
            raw,
            type: NodeType.OTHER,
        };
    });
    if (updateIndex) {
        lines.splice(updateIndex, 1);
        insertionIndex = updateIndex;
    }
    return { lines, insertionIndex };
};
const insertIntoArray = (arr, item, index) => [
    ...arr.slice(0, index),
    item,
    ...arr.slice(index),
];
const createDepNode = (name, artifact) => ({
    type: NodeType.DEPENDENCY,
    raw: createDependencyEntry(artifact, name),
    meta: {
        dependency: name,
        dependencyVersionProperty: `${name}Version`,
    },
});
const addDependencyToSettings = (contents, name, artifact) => {
    const dependencyEntry = createDependencyEntry(artifact, name);
    if (contents.includes(dependencyEntry)) {
        return contents;
    }
    const { lines, insertionIndex } = parseSettings(contents);
    const dep = createDepNode(name, artifact);
    const linesUpdated = insertIntoArray(lines, dep, insertionIndex);
    const serialized = linesUpdated.map((l) => l.raw).join('\n');
    return serialized;
};
const addOrUpdateDependencyToProperties = (contents, id, version) => {
    const keyToUpsert = `${id}Version=`;
    const { lines, insertionIndex } = parseProperties(contents, keyToUpsert);
    const dep = { raw: `${keyToUpsert}${version}` };
    const linesUpdated = insertIntoArray(lines, dep, insertionIndex);
    const serialized = linesUpdated.map((l) => l.raw).join('\n');
    return serialized;
};
const addServerDependency = ({ directory }, { name, artifact }, version) => {
    const settingsPath = `${directory}/server/settings.gradle.kts`;
    const settings = (0, fs_1.readFileSync)(settingsPath, 'utf8');
    const updatedSettings = addDependencyToSettings(settings, name, artifact || name);
    (0, fs_1.writeFileSync)(settingsPath, updatedSettings);
    const propertiesPath = `${directory}/server/gradle.properties`;
    const properties = (0, fs_1.readFileSync)(propertiesPath, 'utf8');
    const updatedProperties = addOrUpdateDependencyToProperties(properties, name, version || name);
    (0, fs_1.writeFileSync)(propertiesPath, updatedProperties);
};
exports.addServerDependency = addServerDependency;
function createDependencyEntry(artifact, name) {
    return ('        dependency("global.genesis:' +
        artifact +
        ':${extra.properties["' +
        name +
        'Version"]}")');
}
//# sourceMappingURL=server.js.map